/*----------------------------------------------------------------------
  File    : ptlset.h
  Contents: prototype-less fuzzy clustering
  Author  : Christian Borgelt
  History : 2007.01.04 file created
            2007.01.14 function ptl_desc added
            2007.01.16 function ptl_parse added
            2007.01.17 first full version completed
            2007.01.23 stored number of objects to consider
            2007.02.27 offset for membership degrees added
            2013.07.18 adapted to definitions ATTID, VALID, TPLID etc.
----------------------------------------------------------------------*/
#ifndef __PTLSET__
#define __PTLSET__
#ifndef SCN_SCAN
#define SCN_SCAN
#endif
#include "scanner.h"
#include "distmat.h"

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
/* --- initialization modes --- */
#define PTL_RANDOM     0        /* initialize memberships randomly */
#define PTL_UNIFORM    1        /* initialize memberships uniformly */
#define PTL_SEED       2        /* randomly select seed objects */

/* --- description modes --- */
#define PTL_TITLE     0x0001    /* print a title (as a comment) */
#define PTL_INFO      0x0002    /* print add. info. (as a comment) */

/*----------------------------------------------------------------------
  Type Definitions
----------------------------------------------------------------------*/
typedef struct {                /* --- prototype-less cluster set --- */
  DIMID  clscnt;                /* number of classes/clusters */
  DIMID  rowcnt;                /* number of tuples in data table */
  DIMID  objcnt;                /* number of objects to consider */
  int    norm;                  /* whether table was normalized */
  double msexp;                 /* exponent for membership degrees */
  double msoff;                 /* offset   for membership degrees */
  double tfexp;                 /* exponent for sum transformation */
  double objval;                /* objective function value */
  double *wgts;                 /* object weights (optional) */
  double **msds;                /* matrix of membership degrees */
  double *buf;                  /* buffer needed for deallocation */
  double **pows;                /* buffer for powers of m.s. degrees */
  double *dists;                /* distance buffer for execution */
  DIMID  *ids;                  /* object identifiers for execution */
} PTLSET;                       /* (prototype-less cluster set) */

/*----------------------------------------------------------------------
  Functions
----------------------------------------------------------------------*/
extern PTLSET* ptl_create  (DIMID rowcnt, DIMID clscnt);
extern void    ptl_delete  (PTLSET *ptl);
extern DIMID   ptl_clscnt  (PTLSET *ptl);
extern DIMID   ptl_rowcnt  (PTLSET *ptl);

extern void    ptl_msexp   (PTLSET *ptl, double msexp, double msoff);
extern void    ptl_tfexp   (PTLSET *ptl, double tfexp);
extern void    ptl_setobjc (PTLSET *ptl, DIMID objcnt);
extern DIMID   ptl_getobjc (PTLSET *ptl);
extern void    ptl_setnorm (PTLSET *ptl, int norm);
extern int     ptl_getnorm (PTLSET *ptl);

extern int     ptl_setwgt  (PTLSET *ptl, DIMID rowid, double wgt);
extern double  ptl_getwgt  (PTLSET *ptl, DIMID rowid);
extern void    ptl_setmsd  (PTLSET *ptl, DIMID rowid, DIMID clsid,
                            double msd);
extern double  ptl_getmsd  (PTLSET *ptl, DIMID rowid, DIMID clsid);

extern void    ptl_init    (PTLSET *ptl, int mode, RANDFN randfn);
extern double  ptl_update  (PTLSET *ptl, DISTMAT *mat,
                            DIMID *order, RANDFN randfn);
extern void    ptl_regular (PTLSET *ptl);
extern double  ptl_objfn   (PTLSET *ptl, DISTMAT *mat);
extern DIMID   ptl_exec    (PTLSET *ptl, double *dists,
                            DIMID *nbids, DIMID nbcnt, double *msds);

extern int     ptl_desc    (PTLSET *ptl, FILE *file,
                            int mode, int maxlen);
extern PTLSET* ptl_parse   (SCANNER *scan);

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
#define ptl_clscnt(p)        ((p)->clscnt)
#define ptl_rowcnt(p)        ((p)->rowcnt)

#define ptl_setobjc(p,x)     ((p)->objcnt = (x))
#define ptl_getobjc(p)       ((p)->objcnt)
#define ptl_setnorm(p,x)     ((p)->norm  = (x))
#define ptl_getnorm(p)       ((p)->norm)

#define ptl_getwgt(p,i)      ((p)->wgts ? (p)->wgts[i] : 1.0)
#define ptl_setmsd(p,r,c,m)  ((p)->msds[r][c] = (m))
#define ptl_getmsd(p,r,c)    ((p)->msds[r][c])

#endif
